"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.isSubscription = exports.isClass = exports.fetchIntrospectionSchema = exports.stripObject = exports.getUrl = exports.getSchemaQuery = exports.getSchemaMutation = exports.getSchemaTypeFields = exports.getParsedSchema = exports.config = exports.registryGet = exports.performSafeRequestGraphql = exports.stripTypenameDefault = exports.stripTypename = exports.performGqlSubscription = exports.performGqlMutation = exports.performGqlQuery = exports.performSafeRequestREST = exports.queryParams = exports.camelToKebab = exports.getApolloClient = exports.defaultRESTHeaders = void 0;
const tslib_1 = require("tslib");
const graphql_1 = require("graphql");
const parser_1 = require("graphql/language/parser");
const omit_deep_lodash_1 = tslib_1.__importDefault(require("omit-deep-lodash"));
const Registry_1 = tslib_1.__importDefault(require("./Registry"));
const UnexpectedException_1 = tslib_1.__importDefault(require("./models/Exceptions/UnexpectedException"));
const makeApolloClients_1 = tslib_1.__importDefault(require("./graphql/makeApolloClients"));
exports.defaultRESTHeaders = {
    'Accept': 'application/json',
    'Content-Type': 'application/json;charset=UTF-8',
};
function getApolloClient(providerName = 'default') {
    return config(providerName).apolloClient || makeApolloClients_1.default(providerName);
}
exports.getApolloClient = getApolloClient;
function camelToKebab(input) {
    return input.replace(/([a-z])([A-Z])/g, '$1-$2').toLowerCase();
}
exports.camelToKebab = camelToKebab;
function queryParams(params, camelToKebabActive = true) {
    const str = [];
    for (const paramsKey in params)
        if (params.hasOwnProperty(paramsKey) && (params[paramsKey] || params[paramsKey] === 0)) {
            const normalized = camelToKebabActive ? camelToKebab(encodeURIComponent(paramsKey)) : encodeURIComponent(paramsKey);
            str.push(`${normalized}=${encodeURIComponent(params[paramsKey])}`);
        }
    return str.join('&');
}
exports.queryParams = queryParams;
async function performSafeRequestREST(url, params = {}, method = 'get', opts = {}) {
    let fullUrl = url;
    let body = {};
    if (method === 'get') {
        const queryEscaped = queryParams(params);
        if (queryEscaped) {
            fullUrl = `${fullUrl}?${queryEscaped}`;
        }
    }
    else {
        body = params;
    }
    return fetch(fullUrl, {
        method: method.toLowerCase(),
        headers: exports.defaultRESTHeaders,
        body: JSON.stringify(body),
        ...opts,
    }).then(response => response.json());
}
exports.performSafeRequestREST = performSafeRequestREST;
function performGqlQuery(query, variables, providerName = 'default') {
    return getApolloClient(providerName).query({
        query,
        variables,
    });
}
exports.performGqlQuery = performGqlQuery;
function performGqlMutation(mutation, variables, providerName = 'default') {
    return getApolloClient(providerName).mutate({
        mutation,
        variables,
    });
}
exports.performGqlMutation = performGqlMutation;
async function performGqlSubscription(subscription, variables, providerName = 'default') {
    return getApolloClient(providerName).subscribe({
        query: subscription,
        variables,
    });
}
exports.performGqlSubscription = performGqlSubscription;
function stripTypename(obj) {
    return config().stripTypename ? config().stripTypename(obj) : stripTypenameDefault(obj);
}
exports.stripTypename = stripTypename;
function stripTypenameDefault(obj) {
    return omit_deep_lodash_1.default(obj, '__typename');
}
exports.stripTypenameDefault = stripTypenameDefault;
async function performSafeRequestGraphql(query, variables = {}, providerName = 'default') {
    const queryName = query.definitions.map(def => def.name).find(def => def.kind === 'Name').value;
    const isQuery = query.definitions.find(def => def.kind === 'OperationDefinition').operation === 'query';
    if (isQuery) {
        return performGqlQuery(query, stripTypename(variables), providerName)
            .then((value) => value.data[queryName]);
    }
    const isSubscription = query.definitions.find(def => def.kind === 'OperationDefinition').operation === 'subscription';
    if (isSubscription) {
        return performGqlSubscription(query, stripTypename(variables), providerName);
    }
    return performGqlMutation(query, stripTypename(variables), providerName)
        .then((value) => value.data[queryName]);
}
exports.performSafeRequestGraphql = performSafeRequestGraphql;
function registryGet(key) {
    return Registry_1.default.getInstance().get(key);
}
exports.registryGet = registryGet;
function config(name = 'default') {
    const defaultConfig = registryGet('Config');
    if (name === 'default') {
        return defaultConfig;
    }
    return defaultConfig.providers.find(config => config.name === name);
}
exports.config = config;
async function getParsedSchema(configName = 'default') {
    const configSchema = config(configName).schema;
    const configSchemaUrl = config(configName).schemaUrl;
    let schema = Registry_1.default.getInstance().get('schema');
    if (!schema && configSchema) {
        schema = configSchema;
    }
    else if (!schema && configSchemaUrl) {
        schema = await fetchIntrospectionSchema(configSchemaUrl)
            .then(graphql_1.buildClientSchema.bind(null))
            .then(graphql_1.printSchema.bind(null))
            .then(parser_1.parse.bind(null));
        Registry_1.default.getInstance().set('schema', schema);
    }
    if (!schema) {
        throw new UnexpectedException_1.default('Configuration error: \'schema\' must be passed as a config key, e.g\n\nimport schema from \'raw-loader!@/../schema.graphql\';\n\n//...\n\nVue.use(VueOOP, {\n  //...,\n  schema,\n})\n\n;');
    }
    return schema;
}
exports.getParsedSchema = getParsedSchema;
async function getSchemaTypeFields(typeName, configName = 'default') {
    return (await getParsedSchema(configName))
        .definitions
        .find(def => (def.name || {}).value === typeName)
        .fields
        .map(f => f.name.value);
}
exports.getSchemaTypeFields = getSchemaTypeFields;
async function getSchemaMutation(mutationName, configName = 'default') {
    return (await getParsedSchema(configName))
        .definitions
        .find(def => (def.name || {}).value === 'Mutation')
        .fields
        .find(def => (def.name || {}).value === mutationName);
}
exports.getSchemaMutation = getSchemaMutation;
async function getSchemaQuery(queryName, configName = 'default') {
    return (await getParsedSchema(configName))
        .definitions
        .find(def => (def.name || {}).value === 'Query')
        .fields
        .find(def => (def.name || {}).value === queryName);
}
exports.getSchemaQuery = getSchemaQuery;
async function getUrl(_opts) {
    const { url, params } = _opts;
    let resolvedUrl = url;
    if (typeof url === 'function') {
        resolvedUrl = await url();
    }
    else {
        resolvedUrl = resolvedUrl.replace(/:([^\s\/?&]+)/gi, (_, m) => {
            const param = params[m];
            const hasParam = param !== undefined;
            if (hasParam) {
                delete params[m];
                return param;
            }
            return m;
        });
    }
    return resolvedUrl;
}
exports.getUrl = getUrl;
function stripObject(obj) {
    return omit_deep_lodash_1.default(obj, 'loading');
}
exports.stripObject = stripObject;
function fetchIntrospectionSchema(url) {
    const body = JSON.stringify({ query: graphql_1.getIntrospectionQuery() });
    return fetch(url, {
        method: 'POST',
        headers: { 'Content-Type': 'application/json' },
        body
    })
        .then(res => res.json())
        .then(res => res.data);
}
exports.fetchIntrospectionSchema = fetchIntrospectionSchema;
exports.isClass = (fn) => /^\s*class/.test(fn.toString());
exports.isSubscription = (data) => data._subscriber;
//# sourceMappingURL=utils.js.map